///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSocketServer_H
#define _CSocketServer_H
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define SSDFLTSENDRECVBUFSZ	65535	// The default size of the common (Packet Pump) send/receive buffer.
#define SSINITSENDRECVSZ	1024	// The initial size of the client send and receive buffers.
#define SSTHREADTIMEOUT		10000	// The number of milliseconds to wait for a thread to start or stop.
#define SSSTOPFORCETIMEOUT	5000	// The number of ms to wait for clients to disconnect on destroy.
#define SSPACKETOVERHEAD	8		// The size of the packet in bytes.
#define SSPACKETHEADER		985746	// Default packet header.

#define SSPACKET_GOOD_PACKET		 1	// OK.
#define SSPACKET_PARTIAL_DATA		 2	// OK.
#define SSPACKET_UNDEFINED		 0	// Error.
#define SSPACKET_INVALID_HEADER	-2	// Error.
#define SSPACKET_INVALID_PACKET	-3	// Error.
#define SSPACKET_MEMORY_ERROR		-4	// Error.
#define SSPACKET_USER_FILTERED	-5	// Error.

extern const char *sSocketServerError[];

#ifdef _DEBUG
#define MS_VC_EXCEPTION 0x406D1388
typedef struct tag_THREADNAME_INFO {
   DWORD dwType;		// Must be 0x1000.
   LPCSTR szName;		// Pointer to name (in user address space).
   DWORD dwThreadID;	// Thread ID (-1 = caller thread).
   DWORD dwFlags;		// Reserved for future use, must be zero.
} THREADNAME_INFO;
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _tag_Basic_Chunk{
	char *sBuffer;
	int iLength;
	int iAlloc;
} BASICHUNK, *LPBASICHUNK;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _TAG_CLIENTCONNECTIONINFO{
	bool bConnected;			// Remote client's connection flag.
	bool bDisconnect;			// Remote client's disconnect flag.
	bool bWasConnect;			// True is the client was connected, false if the client was accepted.
	bool bReserved;				// Reserved client slots will not be given out even if disconnected.
	bool bPacketMode;			// Is the client in packet mode?

	char sPeerAddress[64];		// The IP address of the connected Peer.
	int iPeerPort;				// The IP port of the connected Peer.
	int iPeerID;				// Remote client's ID (For client identification).

	BASICHUNK SendChunk;		// Clients send buffer.
	BASICHUNK RecvChunk;		// Clients receive buffer.

	SOCKET Socket;				// Remote client's socket.
	HANDLE hClientThread;		// The client thread Handle.
	HANDLE hEvent;				// Used for various waiting operations.
	time_t ttLastActivity;		// Used to keep track of client activity.

	VOID *pUserData;			// Generic pointer for storing data per connection (feel free to use).
} CLIENTCONNECTIONINFO, *LPCLIENTCONNECTIONINFO;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CSocketServer {

private: //Delegates:
	typedef bool (*OnAccept)(CSocketServer *pSock, int iClient);
	typedef bool (*OnAcceptConnect)(CSocketServer *pSock, int iClient);
	typedef bool (*OnAfterDePacketize)(CSocketServer *pSock, int iClient, LPBASICHUNK pChunk);
	typedef bool (*OnAfterPacketize)(CSocketServer *pSock, int iClient, LPBASICHUNK pChunk);
	typedef bool (*OnBeforeDePacketize)(CSocketServer *pSock, int iClient, LPBASICHUNK pChunk);
	typedef bool (*OnBeforePacketize)(CSocketServer *pSock, int iClient, LPBASICHUNK pChunk);
	typedef bool (*OnConnect)(CSocketServer *pSock, int iClient);
	typedef bool (*OnRecv)(CSocketServer *pSock, int iClient, LPBASICHUNK pChunk);
	typedef bool (*OnSend)(CSocketServer *pSock, int iClient, LPBASICHUNK pChunk);
	typedef bool (*OnStart)(CSocketServer *pSock, int iListenPort);
	typedef bool (*OnStop)(CSocketServer *pSock);
	typedef void (*ClientHandlerThread)(CSocketServer *pSock, int iClient, LPBASICHUNK pChunk);
	typedef void (*OnBeginClientThread)(CSocketServer *pSock, int iClient, HANDLE hThread);
	typedef void (*OnDisconnect)(CSocketServer *pSock, int iClient);
	typedef void (*OnEndClientThread)(CSocketServer *pSock, int iClient);
	typedef void (*OnError)(CSocketServer *pSock, int iClient, int iErrorNumber, const char *sErrorMsg);

public: //Internal memory handing (**Extremely** thread safe):
	bool Alloc(LPBASICHUNK pChunk);
	bool Alloc(LPBASICHUNK pChunk, int iTotalBytes);
	void *Alloc(const int iSize, const int iCount);

	bool ReAlloc(LPBASICHUNK pChunk, int iTotalBytes);
	void *ReAlloc(void *lpMemory, const int iSize);

	bool Free(LPBASICHUNK pChunk);
	void Free(void *lpMemory);

public: //Helper functions for sending & receiving "Commands"
	bool IsCommand(LPBASICHUNK lpInCmd, const char *sCommand);
	bool ParseCommand(LPBASICHUNK lpInCmd, const char *sCommand, LPBASICHUNK lpOutCmd);
	bool ParseCommand(LPBASICHUNK lpInOutCmd, const char *sCommand);
	bool SendCommand(int iClient, const char *sCommand);
	bool SendCommand(int iClient, const char *sCommand, const char *sData);
	bool SendCommand(int iClient, const char *sCommand, const void *pData, const int iDataLength);
	bool SendCommand(int iClient, const char *sCommand, const void *pData, const int iDataLength, BASICHUNK *pTempChunk);
	bool SendCommand(int iClient, const char *sCommand, const void *sData, const int iDataSz, void *pTemp, int iMaxTempSize);
	int BreakDataFromCommand(const void *pBuffer, const int iBufferLength, const int iCommandLength, void *sOutBuffer, int iMaxOutBufferSz);
	int CompareCommand(const char *sBuffer, const int iBufferSize, const char *sCommand);

public: //General public functions.

	bool IsIPAddress(const char *sString);
	bool ResolveHost(const char *sHostName, char *sOutIP, int iMaxOutSz);

	bool Connect(const char *sHostOrIP, int iRemotePort, int *ioutClient);
	bool ConnectByHost(const char *sHostName, int iRemotePort, int *ioutClient);
	bool ConnectByIP(const char *sIPAddress, int iRemotePort, int *ioutClient);
	bool ConnectAndCreateThread(const char *sHostOrIP, int iRemotePort, int *ioutClient);
	bool ConnectByHostAndCreateThread(const char *sHostName, int iRemotePort, int *ioutClient);
	bool ConnectByIPAndCreateThread(const char *sIPAddress, int iRemotePort, int *ioutClient);
	bool ConnectUseCurrentThread(const char *sHostOrIP, int iRemotePort);
	bool ConnectByHostUseCurrentThread(const char *sHostName, int iRemotePort);
	bool ConnectByIPUseCurrentThread(const char *sIPAddress, int iRemotePort);

	bool GetClientData(int iClient, LPBASICHUNK pChunk);
	bool GetNextClientData(int iClient, LPBASICHUNK pChunk);
	bool GetNextClient(int *iClient);

	bool IsRecvDataWaiting(int iClient);
	bool IsSendDataWaiting(int iClient);

	bool SetNextSendData(int iClient, const void *sData, int iSize);
	bool SetNextSendData(int iClient, const char *sData);
	bool SetNextSendData(int iSourceClientID, int iTargetClientID, const void *sData, int iSize);
	bool SetNextSendData(int iSourceClientID, int iTargetClientID, const char *sData);
	bool SetNextSendData(int iSourceClientID, int iTargetClientID, LPBASICHUNK pChunk);
	bool SetNextSendData(int iClient, LPBASICHUNK pChunk);

	bool SetSendData(int iClient, const char *sData);
	bool SetSendData(int iClient, const void *sData, int iSize);
	bool SetSendData(int iClient, LPBASICHUNK pChunk);

	bool WaitOnDataToBeRecvd(int iClient);
	bool WaitOnDataToBeSent(int iSourceClientID, int iTargetClientID);
	bool WaitOnDataToBeSent(int iClient);
	bool WaitForEmptyRecvBuffer(int iClient);

	bool ResumeClientThread(int iClient);
	bool PauseClientThread(int iClient);

	double MaxInactivityTime(void);
	void MaxInactivityTime(double dSeconds);

	void Reserved(int iClient, bool bReserve);
	bool Reserved(int iClient);
	void AllowNewConnections(bool bAllow);

	void PacketMode(bool bUsePacketing);
	bool PacketMode(void);
	bool PacketMode(int iClient);
	void PacketMode(int iClient, bool bUsePacketing);

	bool PausePumpThread(void);
	bool ResumePumpThread(void);

	HANDLE ClientThreadHandle(int iClient);
	HANDLE ClientEventHandle(int iClient);

	int GetSendBufSz(void);
	int GetRecvBufSz(void);

	int PacketHeader(void);
	void PacketHeader(int iPacketHeader);

	void NextPeerID(int iPeerID);
	int NextPeerID(void);
	int PeerID(int iClientID);
	char *PeerAddress(int iClientID);
	int PeerAddress(int iClientID, char *sOutAddress, int iMaxSz);
	int PeerPort(int iClientID);
	int GetListenPort(void);
	void PacketBufferSize(int iSize);
	int PacketBufferSize(void);

	VOID *UserData(int iClient);
	VOID *UserData(void);
	void UserData(int iClient, VOID *pData);
	void UserData(VOID *pData);

	int CurrentConnections(void);
	int MaxConnections(void);

	bool IsConnected(int iClient);
	bool Disconnect(int iClient);

	bool IsListeningServer(void);
	bool IsActive(void);

	bool Start(void);
	bool Stop(void);
	bool Stop(bool bForce);

	bool Destroy(void);

	bool Initialize(int iListenPort, int iMaxClients, OnError lpOnError); //Low level init.
	bool Initialize(int iListenPort, int iMaxClients, ClientHandlerThread lpClientThread);
	bool Initialize(int iListenPort, int iMaxClients, ClientHandlerThread lpClientThread, OnError lpOnError);
	bool Initialize(OnError lpOnError);
	bool Initialize(ClientHandlerThread lpClientThread);
	bool Initialize(ClientHandlerThread lpClientThread, OnError lpOnError);

	bool TryLock(void);
	void Lock(void);
	void Unlock(void);

private: //Internal events.
	OnAccept pOnAccept;
	OnAcceptConnect pOnAcceptConnect;
	OnAfterDePacketize pOnAfterDePacketize;
	OnAfterPacketize pOnAfterPacketize;
	OnBeforeDePacketize pOnBeforeDePacketize;
	OnBeforePacketize pOnBeforePacketize;
	OnBeginClientThread pOnBeginClientThread;
	OnConnect pOnConnect;
	OnDisconnect pOnDisconnect;
	OnEndClientThread pOnEndClientThread;
	OnError pOnError;
	OnRecv pOnRecv;
	OnSend pOnSend;
	OnStart pOnStart;
	OnStop pOnStop;

public: //Public events.
	void SetClientThread(ClientHandlerThread lpClientThread);
	void SetOnAccept(OnAccept lpOnAccept);
	void SetOnAcceptConnect(OnAcceptConnect lpOnAcceptConnect);
	void SetOnAfterDePacketize(OnAfterDePacketize lpOnAfterDePacketize);
	void SetOnAfterPacketize(OnAfterPacketize lpOnAfterPacketize);
	void SetOnBeforeDePacketize(OnBeforeDePacketize lpOnBeforeDePacketize);
	void SetOnBeforePacketize(OnBeforePacketize lpOnBeforePacketize);
	void SetOnBeginClientThread(OnBeginClientThread lpOnBeginClientThread);
	void SetOnConnect(OnConnect lpOnConnect);
	void SetOnDisconnect(OnDisconnect lpOnDisconnect);
	void SetOnEndClientThread(OnEndClientThread lpOnEndClientThread);
	void SetOnError(OnError lpOnError);
	void SetOnRecv(OnRecv lpOnRecv);
	void SetOnSend(OnSend lpOnSend);
	void SetOnStart(OnStart lpOnStart);
	void SetOnStop(OnStop lpOnStop);

private: //Various internal functions.
	bool p_CloseListenSocket(void);
	bool p_DisconnectClient(int iClient);
	bool p_Accept(void);
	bool p_Connect(const char *sIPAddress, int iRemotePort, int *ioutClient, bool bCreateThread, bool bUseCurrentThread);
	bool p_Receive(int iClient);
	bool p_Send(int iClient);
	bool p_InitializeClientVariables(int iClient);
	bool p_OpenListenSocket(int iListenPort);
	bool p_StartClientThread(int iClient);
	bool p_StartWindowsSocket(void);
	bool p_StopWindowsSocket(void);
	bool p_ThrowError(int iErrorNumber);
	bool p_ThrowError(int iErrorNumber, int iClient);
	bool p_WaitOnClientThreadToExit(int iClient);
	int p_AnalysePacket(int iClient);
	int p_DePacketize(int iClient, BASICHUNK *pChunk);
	int p_GetFreeSocketSlot(void);	// Search for free client "Slot".
	int p_Packetize(int iClient);
	void p_ResetClientSlot(int iClient);

	#ifdef _DEBUG
	#if defined(_MSC_VER) && (_MSC_VER >= 1400)
	void p_SetThreadName(const char *sThreadName);
	#endif
	#endif

public:
	VOID *pUserData;	// Generic pointer for storing data (feel free to use).
	
private: //Various internal variables.
	bool p_bActive;					// Is the server started?
	bool p_bAllowNewConnections;	// Does the class allow new connection (Accept / Connect)?
	int p_iListenPort;				// The port the server was set to listen on (if any).

	CLIENTCONNECTIONINFO *CCI;		// The Conneced Client Information structure.

	int p_iPacketBufferSize;		// The packet buffer size, this can be resized at any time.
	int p_iPacketHeader;			// The packet header that will be used with packeting.
	int p_iCurrentClients;			// The number of clients connected to the server.
	int p_iMaxClients;				// Max clients allowed to connect.
	int p_iNextPeerID;				// The next assigned Client ID.
	int p_iSocketLimit;				// The maximum number of sockets allowed by winsock.
	double p_dMaxInactivityTime;	// The maximum amount of time (in seconds) that a peer is alowed to be idle.
	BASICHUNK p_SendChunk;			// The class send buffer.
	BASICHUNK p_RecvChunk;			// The class receive buffer.

	SOCKET p_ListenSocket;			// Listen socket of local server

	bool p_bIsListeningServer;		// Is this class acting as a listening server?
	bool p_bPacketMode;				// Is the class using "Packet Mode" or "Raw Mode"?

	//------------------------------------------------------------------------
	//The TcpPump routine threading items (Begin)
	//------------------------------------------------------------------------
	friend unsigned __stdcall p_TcpPumpThreadProc(void *pData);

	unsigned int p_TCPPump(void);
	bool p_StartTcpPump(void);
	bool p_StopTcpPump(void);

	bool p_bShutdownTcpPump;
	bool p_bTcpPumpActive;
	HANDLE p_hTcpPumpEvent;
	HANDLE p_hTcpPump;
	//------------------------------------------------------------------------
	//The TcpPump routine threading items (End)
	//------------------------------------------------------------------------

	CRITICAL_SECTION p_TcpPumpLock;
	CRITICAL_SECTION p_HostResolverLock;
	#ifndef _USE_GLOBAL_MEMPOOL
	CRITICAL_SECTION p_MemoryLock;
	#endif

	//The client thread function and delegate.
	friend unsigned __stdcall p_CilentThreadProc(void *pData);
	ClientHandlerThread p_pClientHandlerThread;
};

// Used to pass class data to the client thread
typedef struct _CLIENTTHREADSTARTUPINFO{
	CSocketServer *pSock;
	int iClient;
	bool bIsNewThread;
}CLIENTTHREADSTARTUPINFO, *LPCLIENTTHREADSTARTUPINFO;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
