///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSocketServer_Accept_Cpp
#define _CSocketServer_Accept_Cpp
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define _WIN32_WINNT 0x0500
#define WIN32_LEAN_AND_MEAN

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinSock2.H>
#include <WinSock.H>
#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <Time.H>

#include "CSocketServer.H"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function accepts a new connection from a remote peer.
	This function should not be called directly.
*/
bool CSocketServer::p_Accept(void)
{
	if(!this->p_bAllowNewConnections)
	{
		return false;
	}
	else if(this->p_iCurrentClients >= this->p_iMaxClients)
	{
		return false; //Too many clients already connected.
	}

	int iClient = 0;
	int iLength = 0;
	SOCKET sckLocalSocket = 0;

	SOCKADDR_IN RemoteClientSocketAddress;

	iLength = sizeof(SOCKADDR);
	if((sckLocalSocket = accept(this->p_ListenSocket, (SOCKADDR*)&RemoteClientSocketAddress, &iLength)) == SOCKET_ERROR)
	{
		return this->p_ThrowError(3, iClient);
	}
	else if(sckLocalSocket == 0)
	{
		return this->p_ThrowError(4, iClient);
	}

	if((iClient = this->p_GetFreeSocketSlot()) == -1)
	{
		//WSASendDisconnect(sckLocalSocket, NULL);
		shutdown(sckLocalSocket, SD_BOTH);
		closesocket(sckLocalSocket);
		return this->p_ThrowError(6, iClient);
	}

	this->CCI[iClient].bWasConnect = false;
	this->CCI[iClient].bConnected = true;
	this->CCI[iClient].bDisconnect = false;
	this->CCI[iClient].Socket = sckLocalSocket;

	this->CCI[iClient].iPeerID = this->p_iNextPeerID++;
	this->CCI[iClient].iPeerPort = ntohs(RemoteClientSocketAddress.sin_port);
	strcpy_s(this->CCI[iClient].sPeerAddress,
		sizeof(this->CCI[iClient].sPeerAddress),
		inet_ntoa(RemoteClientSocketAddress.sin_addr));

	if(!this->p_InitializeClientVariables(iClient))
	{
		//WSASendDisconnect(sckLocalSocket, NULL);
		shutdown(sckLocalSocket, SD_BOTH);
		closesocket(sckLocalSocket);

		this->p_ResetClientSlot(iClient);

		return this->p_ThrowError(7, iClient);
	}

	if(this->pOnAccept)
	{
		if(!this->pOnAccept(this, iClient))
		{
			//WSASendDisconnect(sckLocalSocket, NULL);
			shutdown(sckLocalSocket, SD_BOTH);
			closesocket(sckLocalSocket);
			
			this->p_ResetClientSlot(iClient);

			return this->p_ThrowError(8, iClient);
		}
	}

	this->p_iCurrentClients++;

	return this->p_StartClientThread(iClient);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
