///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSocketServer_ClientThread_Cpp
#define _CSocketServer_ClientThread_Cpp
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define _WIN32_WINNT 0x0500
#define WIN32_LEAN_AND_MEAN

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinSock2.H>
#include <WinSock.H>
#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <Time.H>
#include <Process.H>

#include "CSocketServer.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to pause a clients thread.
*/
bool CSocketServer::PauseClientThread(int iClient)
{
	return(SuspendThread(this->CCI[iClient].hClientThread) != -1);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to resume a clients previously paused thread.
*/
bool CSocketServer::ResumeClientThread(int iClient)
{
	return(ResumeThread(this->CCI[iClient].hClientThread) != -1);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns the handle of a clients thread.
*/
HANDLE CSocketServer::ClientThreadHandle(int iClient)
{
	return this->CCI[iClient].hClientThread;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function starts a client thread, it should not be called directly.
*/
bool CSocketServer::p_StartClientThread(int iClient)
{
	//Are we supposed to create client threads?
	if(this->p_pClientHandlerThread)
	{
		char sEventName[255];
		sprintf_s(sEventName, sizeof(sEventName), "CT->%d->%d->%d", iClient, GetTickCount(), rand());
		this->CCI[iClient].hEvent = CreateEvent(NULL, FALSE, FALSE, sEventName);

		CLIENTTHREADSTARTUPINFO CTSI;
		memset(&CTSI, 0, sizeof(CTSI));
		CTSI.pSock = this;
		CTSI.iClient = iClient;
		CTSI.bIsNewThread = true;

		if((this->CCI[iClient].hClientThread = (HANDLE)
			_beginthreadex(NULL, 0, p_CilentThreadProc, (LPVOID) &CTSI, CREATE_SUSPENDED, NULL)) == NULL)
		{
			CloseHandle(this->CCI[iClient].hEvent);
			return this->p_ThrowError(12, iClient);
		}

		SetThreadPriority(this->CCI[iClient].hClientThread, THREAD_PRIORITY_NORMAL);

		ResumeThread(this->CCI[iClient].hClientThread);

		WaitForSingleObject(this->CCI[iClient].hEvent, INFINITE);
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is called to wait on client thread to end when the client is being disconnected.
	It should not be called directly.
*/
bool CSocketServer::p_WaitOnClientThreadToExit(int iClient)
{
	if(this->CCI[iClient].hClientThread) //Does the thread exist?
	{
		DWORD dwExitCode = 0;
		int iWaitCount = 0;
		bool bResult = false;

		if(this->CCI[iClient].hEvent)
		{
			//The thread might be waiting on its hEvent for some reason, signal it.
			SetEvent(this->CCI[iClient].hEvent);
		}

		while((bResult = (GetExitCodeThread(this->CCI[iClient].hClientThread, &dwExitCode) > 0)))
		{
			if(this->TryLock())
			{
				if(dwExitCode != STILL_ACTIVE || iWaitCount == SSTHREADTIMEOUT)
				{
					if(iWaitCount == SSTHREADTIMEOUT)
					{
						TerminateThread(this->CCI[iClient].hClientThread, 1);
						this->p_ThrowError(15, iClient);
						bResult = false;
					}

					//The thread was stopped, trigger the event.
					if(this->pOnEndClientThread)
					{
						this->pOnEndClientThread(this, iClient);
					}
				
					this->Unlock();
					break;
				}
				else{
					if(iWaitCount == 0)
					{
						//If the thread is still active, then we'll give it the
						//	priority it needs to finish up under a high load.
						SetThreadPriority(this->CCI[iClient].hClientThread, THREAD_PRIORITY_ABOVE_NORMAL);
					}
				}
				iWaitCount++;
				this->Unlock();
			}
			Sleep(10);
		}

		if(!bResult)
		{
			this->p_ThrowError(14, iClient);
		}

		//The thread was stopped, trigger the event.
		if(this->pOnEndClientThread)
		{
			this->pOnEndClientThread(this, iClient);
		}

		CloseHandle(this->CCI[iClient].hClientThread);
		CloseHandle(this->CCI[iClient].hEvent);

		this->CCI[iClient].hClientThread = NULL;
		this->CCI[iClient].hEvent = NULL;

		return bResult;
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This is the actual client thread procedure.
*/
unsigned __stdcall p_CilentThreadProc(void *pData)
{
	//---------------------(Initialize Thread [Begin])----------
	CLIENTTHREADSTARTUPINFO *CTSI = (CLIENTTHREADSTARTUPINFO *)pData;
   	CSocketServer *pSock = CTSI->pSock;
   	int iClient = CTSI->iClient;
	bool bIsNewThread = CTSI->bIsNewThread;

	#ifdef _DEBUG
	#if defined(_MSC_VER) && (_MSC_VER >= 1400)
	char sName[255];
	sprintf_s(sName, sizeof(sName), "Client[%d]: %d", iClient, pSock->PeerID(iClient));
	pSock->p_SetThreadName(sName);
	#endif
	#endif

	SetEvent(pSock->CCI[iClient].hEvent);
	//---------------------(Initialize Thread [END])----------

	//The thread has started, trigger the event.
	if(pSock->pOnBeginClientThread)
	{
		pSock->pOnBeginClientThread(pSock, iClient, pSock->CCI[iClient].hClientThread);
	}

	BASICHUNK pChunk;
	if(pSock->Alloc(&pChunk))
	{
		pSock->p_pClientHandlerThread(pSock, iClient, &pChunk);
		pSock->Free(&pChunk);
	}
	else{
		pSock->p_ThrowError(13, iClient);
	}

	if(pSock->IsConnected(iClient))
	{
		pSock->Disconnect(iClient);
	}

	if(bIsNewThread)
	{
		_endthreadex(0);
	}

	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
