///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSocketServer_Command_Cpp
#define _CSocketServer_Command_Cpp
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define _WIN32_WINNT 0x0500
#define WIN32_LEAN_AND_MEAN

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinSock2.H>
#include <WinSock.H>
#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <Time.H>

#include "CSocketServer.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Checks to see if the passed in command data (lpInCmd) is the specified command tag (sCommand),
		if it matches, this function will parse off the data section of the lpInCmd buffer and
		place it in the return command (lpOutCmd).

	If ParseCommand() returns true, you must call CSocketServer::Free() to free the allocated block.
*/

bool CSocketServer::ParseCommand(LPBASICHUNK lpInCmd, const char *sCommand, LPBASICHUNK lpOutCmd)
{
	int iCommandLength = 0;

	if((iCommandLength = CompareCommand(lpInCmd->sBuffer, lpInCmd->iLength, sCommand)))
	{
		int iMemoryRequired = (lpInCmd->iLength - iCommandLength);
		if(iMemoryRequired > 0)
		{
			if(iMemoryRequired > lpOutCmd->iAlloc)
			{
				if(lpOutCmd->iAlloc <= 0)
				{
					lpOutCmd->sBuffer = NULL;
				}

				lpOutCmd->iAlloc = iMemoryRequired + 1;

				if(!(lpOutCmd->sBuffer = (char *) this->ReAlloc(lpOutCmd->sBuffer, lpOutCmd->iAlloc)))
				{
					return false;
				}
			}

			lpOutCmd->iLength = BreakDataFromCommand(lpInCmd->sBuffer,
				lpInCmd->iLength, iCommandLength, lpOutCmd->sBuffer, lpOutCmd->iAlloc);

			lpInCmd->sBuffer[lpOutCmd->iLength] = '\0';

			return true;
		}
		else{
			lpOutCmd->iAlloc = 0;
		}
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSocketServer::IsCommand(LPBASICHUNK lpInCmd, const char *sCommand)
{
	return(this->CompareCommand(lpInCmd->sBuffer, lpInCmd->iLength, sCommand) > 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSocketServer::ParseCommand(LPBASICHUNK lpInOutCmd, const char *sCommand)
{
	return this->ParseCommand(lpInOutCmd, sCommand, lpInOutCmd);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Checks to see if the passed in buffer (sBuffer) starts with the specified command tag (sCommand).

	The function will return the length of the sCommand string.
*/
int CSocketServer::CompareCommand(const char *sBuffer, const int iBufferSize, const char *sCommand)
{
	int iCommandLength = strlen(sCommand);

	if(iCommandLength <= iBufferSize)
	{
		if( _strnicmp(sBuffer, sCommand, iCommandLength) == 0)
		{
			return iCommandLength;
		}
	}
	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Parses off the data section of a command (in pBuffer) and place it in the return buffer (sOutBuffer).

	Returns the length of the data section of the command.
*/
int CSocketServer::BreakDataFromCommand(const void *pBuffer, const int iBufferLength,
										const int iCommandLength, void *sOutBuffer, int iMaxOutBufferSz)
{
	memcpy_s(sOutBuffer, iMaxOutBufferSz - iCommandLength,
		((BYTE *)pBuffer) + iCommandLength, iBufferLength - iCommandLength);
	return iBufferLength - iCommandLength;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Sends a textual command.

	See SetNextSendData() for return values. 
*/
bool CSocketServer::SendCommand(int iClient, const char *sCommand)
{
	return this->SetNextSendData(iClient, sCommand, strlen(sCommand));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends a textual command to command data and send the chunk.

	pTemp must be large enough to hold both sCommand and pData.

	See SetNextSendData() for return values. 
*/
bool CSocketServer::SendCommand(int iClient, const char *sCommand, const char *sData)
{
	return this->SendCommand(iClient, sCommand, sData, strlen(sData));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends a textual command to command data and send the chunk.

	pTemp must be large enough to hold both sCommand and pData.

	See SetNextSendData() for return values. 
*/
bool CSocketServer::SendCommand(int iClient, const char *sCommand,
								const void *pData, const int iDataLength)
{
	BASICHUNK Temp;

	memset(&Temp, 0, sizeof(Temp));
	bool bResult = this->SendCommand(iClient, sCommand, pData, iDataLength, &Temp);
	this->Free(&Temp);

	return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Appends a textual command to command data and send the chunk.

	pTemp must be large enough to hold both sCommand and pData.

	See SetNextSendData() for return values. 
*/
bool CSocketServer::SendCommand(int iClient, const char *sCommand,
								const void *pData, const int iDataLength,
								BASICHUNK *pTempChunk)
{
	int iCommandLength = strlen(sCommand);

	pTempChunk->iLength = iCommandLength + iDataLength;

	if(pTempChunk->iAlloc <= pTempChunk->iLength)
	{
		if(!this->ReAlloc(pTempChunk, pTempChunk->iLength + 1))
		{
			pTempChunk->iLength = 0;
			return false;
		}
	}

	memcpy_s(pTempChunk->sBuffer, pTempChunk->iAlloc, sCommand, iCommandLength);

	memcpy_s(pTempChunk->sBuffer + iCommandLength,
		pTempChunk->iAlloc - iCommandLength, pData, iDataLength);

	pTempChunk->sBuffer[pTempChunk->iLength] = '\0';

	return this->SetNextSendData(iClient, pTempChunk);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
