///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSocketServer_Connect_Cpp
#define _CSocketServer_Connect_Cpp
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define _WIN32_WINNT 0x0500
#define WIN32_LEAN_AND_MEAN

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinSock2.H>
#include <WinSock.H>
#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <Time.H>

#include "CSocketServer.H"

/*
	Establishes a new connection to a remote peer.

	This Function is not multi-thread safe, do not use this function directly.
*/
bool CSocketServer::p_Connect(const char *sIPAddress, int iRemotePort, int *ioutClient,
								bool bCreateThread, bool bUseCurrentThread)
{
	if(!this->p_bAllowNewConnections)
	{
		return false;
	}

	//Try to aquire the lock.
	for(int iRetry = 0; !this->TryLock(); iRetry++)
	{
		if(iRetry == 5000)
		{
			return false;
		}
		Sleep(1);
	}

	SOCKADDR_IN RemoteServerSocketAddress;

	SOCKET sckLocalSocket = 0;

	if(this->p_iCurrentClients == this->p_iMaxClients)
	{
		this->Unlock();
		return this->p_ThrowError(26);
	}

	RemoteServerSocketAddress.sin_family	  = AF_INET;
	RemoteServerSocketAddress.sin_port		= htons(iRemotePort);
	RemoteServerSocketAddress.sin_addr.s_addr = inet_addr(sIPAddress);

	sckLocalSocket = socket(AF_INET, SOCK_STREAM, 0);

	if(connect(sckLocalSocket,
		(struct sockaddr *) &RemoteServerSocketAddress,
		sizeof(RemoteServerSocketAddress)) == SOCKET_ERROR)
	{
		//WSASendDisconnect(sckLocalSocket, NULL);
		shutdown(sckLocalSocket, SD_BOTH);
		closesocket(sckLocalSocket);
		this->Unlock();
		return false;
	}

	if((*ioutClient = this->p_GetFreeSocketSlot()) == -1)
	{
		//WSASendDisconnect(sckLocalSocket, NULL);
		shutdown(sckLocalSocket, SD_BOTH);
		closesocket(sckLocalSocket);
		this->Unlock();
		return this->p_ThrowError(27);
	}

	this->CCI[*ioutClient].bWasConnect = true;
	this->CCI[*ioutClient].bConnected = true;
	this->CCI[*ioutClient].bDisconnect = false;
	this->CCI[*ioutClient].Socket = sckLocalSocket;
	this->CCI[*ioutClient].hClientThread = NULL;

	this->CCI[*ioutClient].iPeerID = this->p_iNextPeerID++;
	this->CCI[*ioutClient].iPeerPort = iRemotePort;
	strcpy_s(this->CCI[*ioutClient].sPeerAddress,
		sizeof(this->CCI[*ioutClient].sPeerAddress),
		sIPAddress);

	if(!this->p_InitializeClientVariables(*ioutClient))
	{
		//WSASendDisconnect(sckLocalSocket, NULL);
		shutdown(sckLocalSocket, SD_BOTH);
		closesocket(sckLocalSocket);

		this->p_ResetClientSlot(*ioutClient);

		this->Unlock();
		return this->p_ThrowError(28);
	}

	if(this->pOnConnect)
	{
		if(!this->pOnConnect(this, *ioutClient))
		{
			//WSASendDisconnect(sckLocalSocket, NULL);
			shutdown(sckLocalSocket, SD_BOTH);
			closesocket(sckLocalSocket);

			this->p_ResetClientSlot(*ioutClient);

			this->Unlock();
			return this->p_ThrowError(31, *ioutClient);
		}
	}

	this->p_iCurrentClients++;

	if(bCreateThread)
	{
		bool bResult = this->p_StartClientThread(*ioutClient);
		this->Unlock();
		return bResult;
	}
	else if(bUseCurrentThread)
	{
		char sEventName[255];
		sprintf_s(sEventName, sizeof(sEventName),
			"CT->%d->%d->%d", ((int)*ioutClient), GetTickCount(), rand());
		this->CCI[((int)*ioutClient)].hEvent = CreateEvent(NULL, FALSE, FALSE, sEventName);

		CLIENTTHREADSTARTUPINFO CTSI;
		memset(&CTSI, 0, sizeof(CTSI));
		CTSI.pSock = this;
		CTSI.iClient = ((int)*ioutClient);
		CTSI.bIsNewThread = false;

		this->Unlock();
		p_CilentThreadProc(&CTSI);
		return true;
	}

	this->Unlock();
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Establishes a new connection to a remote peer by IP or Hostname then returns.
*/
bool CSocketServer::Connect(const char *sHostOrIP, int iRemotePort, int *ioutClient)
{
	if(this->IsIPAddress(sHostOrIP))
	{
		return this->ConnectByIP(sHostOrIP, iRemotePort, ioutClient);
	}
	else{
		return this->ConnectByHost(sHostOrIP, iRemotePort, ioutClient);
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Establishes a new connection to a remote peer by Hostname then returns.
*/
bool CSocketServer::ConnectByHost(const char *sHostName, int iRemotePort, int *ioutClient)
{
	char sIPAddress[16];
	if(this->ResolveHost(sHostName, sIPAddress, sizeof(sIPAddress)))
	{
		return this->ConnectByIP(sIPAddress, iRemotePort, ioutClient);
	}
	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Establishes a new connection to a remote peer by IP then returns.
*/
bool CSocketServer::ConnectByIP(const char *sIPAddress, int iRemotePort, int *ioutClient)
{
	return this->p_Connect(sIPAddress, iRemotePort, ioutClient, false, false);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Establishes a new connection to a remote peer by IP or Hostname,
		creates a new thread to handle the client's transactions then returns.
*/
bool CSocketServer::ConnectAndCreateThread(const char *sHostOrIP, int iRemotePort, int *ioutClient)
{
	if(this->IsIPAddress(sHostOrIP))
	{
		return this->ConnectByIPAndCreateThread(sHostOrIP, iRemotePort, ioutClient);
	}
	else{
		return this->ConnectByHostAndCreateThread(sHostOrIP, iRemotePort, ioutClient);
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Establishes a new connection to a remote peer by Hostname,
		creates a new thread to handle the client's transactions then returns.
*/
bool CSocketServer::ConnectByHostAndCreateThread(const char *sHostName, int iRemotePort, int *ioutClient)
{
	char sIPAddress[16];
	if(this->ResolveHost(sHostName, sIPAddress, sizeof(sIPAddress)))
	{
		return this->ConnectByIPAndCreateThread(sIPAddress, iRemotePort, ioutClient);
	}
	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Establishes a new connection to a remote peer by IP,
		creates a new thread to handle the client's transactions then returns.
*/

bool CSocketServer::ConnectByIPAndCreateThread(const char *sIPAddress, int iRemotePort, int *ioutClient)
{
	return this->p_Connect(sIPAddress, iRemotePort, ioutClient, true, false);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Establishes a new connection to a remote peer by IP or Hostname,
		uses the current thread to handle the client's transactions
		then returns after the client disconnects.
*/
bool CSocketServer::ConnectUseCurrentThread(const char *sHostOrIP, int iRemotePort)
{
	if(this->IsIPAddress(sHostOrIP))
	{
		return this->ConnectByIPUseCurrentThread(sHostOrIP, iRemotePort);
	}
	else{
		return this->ConnectByHostUseCurrentThread(sHostOrIP, iRemotePort);
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Establishes a new connection to a remote peer by Hostname,
		uses the current thread to handle the client's transactions
		then returns after the client disconnects.
*/
bool CSocketServer::ConnectByHostUseCurrentThread(const char *sHostName, int iRemotePort)
{
	char sIPAddress[16];
	if(this->ResolveHost(sHostName, sIPAddress, sizeof(sIPAddress)))
	{
		return this->ConnectByIPUseCurrentThread(sIPAddress, iRemotePort);
	}
	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Establishes a new connection to a remote peer by IP,
		uses the current thread to handle the client's transactions
		then returns after the client disconnects.
*/
bool CSocketServer::ConnectByIPUseCurrentThread(const char *sIPAddress, int iRemotePort)
{
	int iClient = 0;
	return this->p_Connect(sIPAddress, iRemotePort, &iClient, false, true);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
