///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSocketServer_Initialize_Cpp
#define _CSocketServer_Initialize_Cpp
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define _WIN32_WINNT 0x0500
#define WIN32_LEAN_AND_MEAN

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinSock2.H>
#include <WinSock.H>
#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <Time.H>

#include "CSocketServer.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to Initialize all of the class variables for a connecting client.
*/
bool CSocketServer::Initialize(ClientHandlerThread lpClientThread)
{
	bool bResult = this->Initialize(0, 1, (OnError)NULL);
	if(bResult)
	{
		this->SetClientThread(lpClientThread);
	}
	return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to Initialize all of the class variables for a connecting client.
*/
bool CSocketServer::Initialize(ClientHandlerThread lpClientThread, OnError lpOnError)
{
	bool bResult = this->Initialize(0, 1, lpOnError);
	if(bResult)
	{
		this->SetClientThread(lpClientThread);
	}
	return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to Initialize all of the class variables for a connecting client.
*/
bool CSocketServer::Initialize(OnError lpOnError)
{
	return this->Initialize(0, 1, lpOnError);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to Initialize all of the class variables for a listening server.
*/
bool CSocketServer::Initialize(int iListenPort,
							   int iMaxClients, ClientHandlerThread lpClientThread)
{
	bool bResult = this->Initialize(iListenPort, iMaxClients, (OnError)NULL);
	if(bResult)
	{
		this->SetClientThread(lpClientThread);
	}
	return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to Initialize all of the class variables for a listening server.
*/
bool CSocketServer::Initialize(int iListenPort, int iMaxClients,
							   ClientHandlerThread lpClientThread, OnError lpOnError)
{
	bool bResult = this->Initialize(iListenPort, iMaxClients, (OnError)lpOnError);
	this->SetClientThread(lpClientThread);
	return bResult;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to Initialize all of the class variables for a listening server.
*/
bool CSocketServer::Initialize(int iListenPort, int iMaxClients, OnError lpOnError)
{
	srand(GetTickCount());

	this->p_bActive = false;

	//Defaut event handlers.
	this->pOnAccept = NULL;
	this->pOnAcceptConnect = NULL;
	this->pOnAfterDePacketize = NULL;
	this->pOnAfterPacketize = NULL;
	this->pOnBeforeDePacketize = NULL;
	this->pOnBeforePacketize = NULL;
	this->pOnBeginClientThread = NULL;
	this->pOnConnect = NULL;
	this->pOnDisconnect = NULL;
	this->pOnEndClientThread = NULL;
	this->pOnError = lpOnError;
	this->pOnRecv = NULL;
	this->pOnSend = NULL;
	this->pOnStart = NULL;
	this->pOnStop = NULL;

	if(!this->p_StartWindowsSocket())
	{
		return false;
	}

	if(iMaxClients > this->p_iSocketLimit)
	{
		this->p_StopWindowsSocket();
		return this->p_ThrowError(32);
	}

	if(iMaxClients > FD_SETSIZE)
	{
		this->p_StopWindowsSocket();
		return this->p_ThrowError(29);
	}

	this->p_pClientHandlerThread = NULL;
	this->p_iListenPort = iListenPort;
	this->p_iMaxClients = iMaxClients;
	this->p_iCurrentClients = 0;
	this->p_iNextPeerID = 0;
	this->p_iPacketBufferSize = SSDFLTSENDRECVBUFSZ;
	this->p_iPacketHeader = SSPACKETHEADER;
	this->p_dMaxInactivityTime = 0;
	this->p_bIsListeningServer = false;
	this->p_bAllowNewConnections = false;
	this->p_bPacketMode = true;

	//Initialize the Critical Section objects.
	InitializeCriticalSection(&this->p_TcpPumpLock);
	InitializeCriticalSection(&this->p_HostResolverLock);
	#ifndef _USE_GLOBAL_MEMPOOL
		InitializeCriticalSection(&this->p_MemoryLock);
	#endif

	//Allocate memory for all of our class variables.
	if(!this->Alloc(&this->p_SendChunk, this->p_iPacketBufferSize))
	{
		this->p_StopWindowsSocket();
		return false;
	}

	if(!this->Alloc(&this->p_RecvChunk, this->p_iPacketBufferSize))
	{
		this->Free(&this->p_SendChunk);
		this->p_StopWindowsSocket();
		return false;
	}

	if((this->CCI = (CLIENTCONNECTIONINFO *) this->Alloc(sizeof(CLIENTCONNECTIONINFO), iMaxClients)) == NULL)
	{
		this->Free(&this->p_SendChunk);
		this->Free(&this->p_RecvChunk);
		this->p_StopWindowsSocket();
		return false;
	}

	//Set all of our Client variables to their default values.
	for(int iClient = 0; iClient < iMaxClients; iClient++)
	{
		this->p_ResetClientSlot(iClient);
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
