///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSocketServer_Internal_Cpp
#define _CSocketServer_Internal_Cpp
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define _WIN32_WINNT 0x0500
#define WIN32_LEAN_AND_MEAN

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinSock2.H>
#include <WinSock.H>
#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <Time.H>

#include "CSocketServer.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function sets up a socket to listen for incomming connections.
	This function should not be called directly.
*/
bool CSocketServer::p_OpenListenSocket(int iListenPort)
{
	SOCKADDR_IN LocalListenSocketAddress; // Socket address of local server

	// Fill In The Address Structure For Local Server
	LocalListenSocketAddress.sin_family	  = AF_INET;			   // Address Family.
	LocalListenSocketAddress.sin_addr.s_addr = INADDR_ANY;		 // Let socket Library Assign Address.
	LocalListenSocketAddress.sin_port		= htons(iListenPort); // Port Number.

	// Create A TCP/IP Stream Socket To "Listen" with.
	if((this->p_ListenSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == INVALID_SOCKET)
	{
		return this->p_ThrowError(16);
	}

	// Bind The Name To The Socket.
	if(bind(this->p_ListenSocket, (SOCKADDR*)&LocalListenSocketAddress, sizeof(struct sockaddr)) == SOCKET_ERROR)
	{
		closesocket(this->p_ListenSocket);
		return this->p_ThrowError(17);
	}

	// Set The Socket To Listen.
	if(listen(this->p_ListenSocket, SOMAXCONN) == SOCKET_ERROR)
	{
		closesocket(this->p_ListenSocket);
		return this->p_ThrowError(18);
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function closes down the listening socket.
	This function should not be called directly.
*/
bool CSocketServer::p_CloseListenSocket(void)
{
	if(closesocket(this->p_ListenSocket) == SOCKET_ERROR)
	{
		return false;
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is called to disconnect a client.
	This functions return value has no meaning.
	This function should not be called directly.
*/
bool CSocketServer::p_DisconnectClient(int iClient)
{
	this->p_WaitOnClientThreadToExit(iClient);

	this->Lock(); //We MUST lock here, this cannot be a TryLock.

	//The client is not connected.
	if(!this->CCI[iClient].bConnected)
	{
		this->CCI[iClient].bDisconnect = false;
		return true;
	}

	this->CCI[iClient].bConnected = false;
	this->CCI[iClient].bDisconnect = false;

	this->p_iCurrentClients--;

	//WSASendDisconnect(this->CCI[iClient].Socket, NULL);
	shutdown(this->CCI[iClient].Socket, SD_BOTH);
	closesocket(this->CCI[iClient].Socket);

	if(this->pOnDisconnect)
	{
		this->pOnDisconnect(this, iClient);
	}
	
	this->Free(&this->CCI[iClient].SendChunk);
	this->Free(&this->CCI[iClient].RecvChunk);

	this->p_ResetClientSlot(iClient);

	this->Unlock();

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is called for each connecting client just
	before the client's bcConnected flag is set to TRUE.

	If this function returns FALSE the client wll not be accepted or connected.
	This function should not be called directly.
*/
bool CSocketServer::p_InitializeClientVariables(int iClient)
{
	if(this->pOnAcceptConnect)
	{
		if(!this->pOnAcceptConnect(this, iClient))
		{
			return false;
		}
	}
	
	//Allocate RAM for the clients send buffer.
	if(!this->Alloc(&this->CCI[iClient].SendChunk))
	{
		return false;
	}

	//Allocate RAM for the clients receive buffer.
	if(!this->Alloc(&this->CCI[iClient].RecvChunk))
	{
		return false;
	}
   
	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns true if the passed in error number is OK, otherwise this function
		will call the OnError event then return false.
	This function should not be called directly.
*/
bool CSocketServer::p_ThrowError(int iErrorNumber, int iClient)
{
	if(!iErrorNumber)
	{
		return true;
	}
	else{
		if(this->pOnError)
		{
			this->pOnError(this, iClient, iErrorNumber, sSocketServerError[iErrorNumber]);
		}
		return false;
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Returns true if the passed in error number is OK, otherwise this function
		will call the OnError event then return false.
	This function should not be called directly.
*/
bool CSocketServer::p_ThrowError(int iErrorNumber)
{
	return this->p_ThrowError(iErrorNumber, -1);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function searched the bcConnected[] array to find a free client slot.

	If this function returns -1 the client wll not be accepted or connected.
	This function should not be called directly.
*/
int CSocketServer::p_GetFreeSocketSlot(void)
{
	for(int iClient = 0; iClient < this->p_iMaxClients; iClient++)
	{
		if(!this->CCI[iClient].bConnected && !this->CCI[iClient].bDisconnect && !this->CCI[iClient].bReserved)
		{
			this->p_ResetClientSlot(iClient);
			return iClient;
		}
	}

	return -1;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to clear the Clients position in the CLIENTCONNECTIONINFO structure.
	The memset() is strictly for security purposes.

	This function should not be called directly.
*/
void CSocketServer::p_ResetClientSlot(int iClient)
{
	LPVOID lpUserData = this->CCI[iClient].pUserData;
	memset(&this->CCI[iClient], 0, sizeof(CLIENTCONNECTIONINFO));
	this->CCI[iClient].pUserData = lpUserData;
	this->CCI[iClient].bPacketMode = this->p_bPacketMode;
	time(&this->CCI[iClient].ttLastActivity);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef _DEBUG
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
void CSocketServer::p_SetThreadName(const char *sThreadName)
{
   THREADNAME_INFO info;
   info.dwType = 0x1000;
   info.szName = sThreadName;
   info.dwThreadID = GetCurrentThreadId();
   info.dwFlags = 0;

   __try
   {
	  RaiseException( MS_VC_EXCEPTION, 0, sizeof(info)/sizeof(ULONG_PTR), (ULONG_PTR*)&info );
   }
   __except(EXCEPTION_EXECUTE_HANDLER)
   {
   }
}
#endif
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
