///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSocketServer_Memory_Cpp
#define _CSocketServer_Memory_Cpp
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define _WIN32_WINNT 0x0500
#define WIN32_LEAN_AND_MEAN 

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinSock2.H>
#include <WinSock.H>
#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <Time.H>

#include "CSocketServer.H"

#ifdef _USE_GLOBAL_MEMPOOL
#include "../CMemPool/CMemPool.H"
extern CMemPool *pMem; //gMem must be defined and initalized elsewhere.
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Allocates a chunk of RAM of specified size.
*/
bool CSocketServer::Alloc(LPBASICHUNK pChunk, int iTotalBytes)
{
	memset(pChunk, 0, sizeof(BASICHUNK));
	if(iTotalBytes > 0)
	{
		pChunk->iAlloc = iTotalBytes;
		if(!(pChunk->sBuffer = (char *) this->Alloc(pChunk->iAlloc, 1)))
		{
			return false;
		}
	}
	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Allocates a chunk of RAM of default size.
*/
bool CSocketServer::Alloc(LPBASICHUNK pChunk)
{
	return this->Alloc(pChunk, SSINITSENDRECVSZ);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Allocates additional RAM of specified size to a buffer.
*/
void *CSocketServer::ReAlloc(void *lpMemory, const int iSize)
{
	#ifdef _USE_GLOBAL_MEMPOOL
		char *sMemory = (char *) pMem->ReAllocate(lpMemory, sizeof(char), iSize);
	#else
		EnterCriticalSection(&this->p_MemoryLock);
		char *sMemory = (char *) realloc(lpMemory, iSize);
		LeaveCriticalSection(&this->p_MemoryLock);
	#endif

	return sMemory;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Allocates additional RAM of specified size to a chunk.
*/
bool CSocketServer::ReAlloc(LPBASICHUNK pChunk, int iTotalBytes)
{
	if(iTotalBytes > 0)
	{
		pChunk->iAlloc = iTotalBytes;
		if(!(pChunk->sBuffer = (char *) this->ReAlloc(pChunk->sBuffer, pChunk->iAlloc)))
		{
			return false;
		}
	}
	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Allocates a buffer of RAM of specified size.
*/
void *CSocketServer::Alloc(const int iSize, const int iCount)
{
	#ifdef _USE_GLOBAL_MEMPOOL
		char *sMemory = (char *) pMem->Allocate(iCount, iSize);
	#else
		EnterCriticalSection(&this->p_MemoryLock);
		char *sMemory = (char *) calloc(iCount, iSize);
		LeaveCriticalSection(&this->p_MemoryLock);
	#endif
	return sMemory;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Frees a buffer of RAM.
*/
void CSocketServer::Free(void *lpMemory)
{
	#ifdef _USE_GLOBAL_MEMPOOL
		pMem->Free(lpMemory);
	#else
		EnterCriticalSection(&this->p_MemoryLock);
		free(lpMemory);
		LeaveCriticalSection(&this->p_MemoryLock);
	#endif
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Frees a chunk of RAM.
*/
bool CSocketServer::Free(LPBASICHUNK pChunk)
{
	if(pChunk->sBuffer)
	{
		this->Free(pChunk->sBuffer);
		memset(pChunk, 0, sizeof(BASICHUNK));
		return true;
	}
	memset(pChunk, 0, sizeof(BASICHUNK));
	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
