///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSocketServer_Packetize_Cpp
#define _CSocketServer_Packetize_Cpp
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define _WIN32_WINNT 0x0500
#define WIN32_LEAN_AND_MEAN 

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinSock2.H>
#include <WinSock.H>
#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <Time.H>

#include "CSocketServer.H"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Creates a packet from a clients pending send data. This function should not be called directly.
*/
int CSocketServer::p_Packetize(int iClient)
{
	if(!this->CCI[iClient].bPacketMode)
	{
		if(this->p_SendChunk.iAlloc <= this->CCI[iClient].SendChunk.iLength)
		{
			if(!this->ReAlloc(&this->p_SendChunk, this->CCI[iClient].SendChunk.iLength + 1))
			{
				return SSPACKET_MEMORY_ERROR;
			}
		}

		//Copy the packet data to the output block.
		memcpy_s(this->p_SendChunk.sBuffer,
			this->p_SendChunk.iAlloc,
			this->CCI[iClient].SendChunk.sBuffer,
			this->CCI[iClient].SendChunk.iLength);

		this->p_SendChunk.iLength = this->CCI[iClient].SendChunk.iLength;
		this->p_SendChunk.sBuffer[this->p_SendChunk.iLength] = '\0';
		return this->p_SendChunk.iLength;
	}

	if(this->pOnBeforePacketize)
	{
		if(!this->pOnBeforePacketize(this, iClient, &this->CCI[iClient].SendChunk))
		{
			return SSPACKET_USER_FILTERED;
		}
	}

	if(this->p_SendChunk.iAlloc <= this->CCI[iClient].SendChunk.iLength + SSPACKETOVERHEAD)
	{
		if(!this->ReAlloc(&this->p_SendChunk, this->CCI[iClient].SendChunk.iLength + SSPACKETOVERHEAD + 1))
		{
			return SSPACKET_MEMORY_ERROR;
		}
	}

	//Append the packet header.
	memcpy_s(this->p_SendChunk.sBuffer,
		this->p_SendChunk.iAlloc,
		&this->p_iPacketHeader,
		sizeof(this->p_iPacketHeader));

	//Append the full packet size.
	memcpy_s(this->p_SendChunk.sBuffer + sizeof(this->p_iPacketHeader),
		this->p_SendChunk.iAlloc,
		&this->CCI[iClient].SendChunk.iLength,
		sizeof(this->CCI[iClient].SendChunk.iLength));

	//Append the packet data.
	memcpy_s(this->p_SendChunk.sBuffer + SSPACKETOVERHEAD,
		this->p_SendChunk.iAlloc,
		this->CCI[iClient].SendChunk.sBuffer,
		this->CCI[iClient].SendChunk.iLength);

	this->p_SendChunk.iLength = this->CCI[iClient].SendChunk.iLength + SSPACKETOVERHEAD;
	this->p_SendChunk.sBuffer[this->p_SendChunk.iLength] = '\0';

	if(this->pOnAfterPacketize)
	{
		if(!this->pOnAfterPacketize(this, iClient, &this->p_SendChunk))
		{
			return SSPACKET_USER_FILTERED;
		}
	}

	return this->p_SendChunk.iLength; // The total length of the packet.
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	Analyses received client data to determine if they are full, partial or invalid packets.
	This function should not be called directly.
*/
int CSocketServer::p_AnalysePacket(int iClient)
{
	int iPacketHeader = 0;
	int iReportedLength = 0;

	if(!this->CCI[iClient].bPacketMode)
	{
		return SSPACKET_GOOD_PACKET;
	}

	//Read the packet header.
	memcpy_s(&iPacketHeader, sizeof(iPacketHeader),
		this->CCI[iClient].RecvChunk.sBuffer,
		sizeof(iPacketHeader));

	if(iPacketHeader != this->p_iPacketHeader)
	{
		return SSPACKET_INVALID_HEADER;
	}

	//Read the size of the full packet (we may have received less data, which is OK).
	memcpy_s(&iReportedLength, sizeof(iReportedLength),
		this->CCI[iClient].RecvChunk.sBuffer + sizeof(iPacketHeader), sizeof(iReportedLength));

	//If the data chunk is less than or equal to zero bytes, the packet must be rejected.
	if(iReportedLength <= 0)
	{
		return SSPACKET_INVALID_PACKET;
	}

	if((iReportedLength + SSPACKETOVERHEAD) > this->CCI[iClient].RecvChunk.iLength)
	{
		//The receive buffer currently contains a partial packet.
		//More data will have to be received before this data can be processed.
		return SSPACKET_PARTIAL_DATA;
	}
	
	return SSPACKET_GOOD_PACKET;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function retreves the data segment from a clients packet.
	This function should not be called directly.
*/
int CSocketServer::p_DePacketize(int iClient, BASICHUNK *pChunk)
{
	if(!this->CCI[iClient].bPacketMode)
	{
		if(pChunk->iAlloc <= this->CCI[iClient].RecvChunk.iLength)
		{
			if(!this->ReAlloc(pChunk, this->CCI[iClient].RecvChunk.iLength + 1))
			{
				return SSPACKET_MEMORY_ERROR;
			}
		}

		//Copy the packet data to the output block.
		memcpy_s(pChunk->sBuffer, pChunk->iAlloc,
			this->CCI[iClient].RecvChunk.sBuffer,
			this->CCI[iClient].RecvChunk.iLength);

		pChunk->iLength = this->CCI[iClient].RecvChunk.iLength;
		pChunk->sBuffer[pChunk->iLength] = '\0';

		this->CCI[iClient].RecvChunk.iLength = 0;

		return SSPACKET_GOOD_PACKET;
	}

	int iPacketHeader = 0;
	int iReportedLength = 0;

	if(this->pOnBeforeDePacketize)
	{
		if(!this->pOnBeforeDePacketize(this, iClient, &this->CCI[iClient].RecvChunk))
		{
			return SSPACKET_USER_FILTERED;
		}
	}

	//Read the packet header.
	memcpy_s(&iPacketHeader, sizeof(iPacketHeader),
		this->CCI[iClient].RecvChunk.sBuffer,
		sizeof(iPacketHeader));

	if(iPacketHeader != this->p_iPacketHeader)
	{
		return SSPACKET_INVALID_HEADER;
	}

	//Read the size of the full packet (we may have received less data, which is OK).
	memcpy_s(&iReportedLength, sizeof(iReportedLength),
		this->CCI[iClient].RecvChunk.sBuffer + sizeof(iPacketHeader), sizeof(iReportedLength));

	//If the data chunk is less than or equal to zero bytes, the packet must be rejected.
	if(iReportedLength <= 0)
	{
		return SSPACKET_INVALID_PACKET;
	}

	if((iReportedLength + SSPACKETOVERHEAD) > this->CCI[iClient].RecvChunk.iLength)
	{
		//The receive buffer currently contains a partial packet.
		//More data will have to be received before this data can be processed.
		return SSPACKET_PARTIAL_DATA;
	}
	
	if(pChunk->iAlloc <= iReportedLength)
	{
		if(!this->ReAlloc(pChunk, iReportedLength + 1))
		{
			return SSPACKET_MEMORY_ERROR;
		}
	}
	
	//Copy the packet data to the output block.
	memcpy_s(pChunk->sBuffer,
		pChunk->iAlloc,
		this->CCI[iClient].RecvChunk.sBuffer + SSPACKETOVERHEAD,
		iReportedLength);

	pChunk->iLength = iReportedLength;
	pChunk->sBuffer[pChunk->iLength] = '\0';

	if((this->CCI[iClient].RecvChunk.iLength -= (pChunk->iLength + SSPACKETOVERHEAD)) > 0)
	{
		//Reposition the remaining class buffer data to position zero.
		memcpy_s(this->CCI[iClient].RecvChunk.sBuffer,
			this->CCI[iClient].RecvChunk.iLength,
			this->CCI[iClient].RecvChunk.sBuffer + (pChunk->iLength + SSPACKETOVERHEAD),
			this->CCI[iClient].RecvChunk.iLength);
	}

	if(this->pOnAfterDePacketize)
	{
		if(!this->pOnAfterDePacketize(this, iClient, pChunk))
		{
			return SSPACKET_USER_FILTERED;
		}
	}

	return SSPACKET_GOOD_PACKET;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
