///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSocketServer_PumpThread_Cpp
#define _CSocketServer_PumpThread_Cpp
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define _WIN32_WINNT 0x0500
#define WIN32_LEAN_AND_MEAN 

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinSock2.H>
#include <WinSock.H>
#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <Time.H>
#include <Process.H>

#include "CSocketServer.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to pause the data pump thread.
*/
bool CSocketServer::PausePumpThread(void)
{
	return(SuspendThread(this->p_hTcpPump) != -1);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to resume the previously paused data pump thread.
*/
bool CSocketServer::ResumePumpThread(void)
{
	return(ResumeThread(this->p_hTcpPump) != -1);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to start the data pump thread. It should not be called directly.
*/
bool CSocketServer::p_StartTcpPump(void)
{
	char sEventName[255];
	sprintf_s(sEventName, sizeof(sEventName), "TP->%d->%d", GetTickCount(), rand());
	p_hTcpPumpEvent = CreateEvent(NULL, TRUE, FALSE, sEventName);

	if((this->p_hTcpPump = (HANDLE) _beginthreadex(NULL, 0,
		p_TcpPumpThreadProc, (LPVOID) this, CREATE_SUSPENDED, NULL)) == NULL)
	{
		CloseHandle(p_hTcpPumpEvent);
		return this->p_ThrowError(23);
	}

	SetThreadPriority(this->p_hTcpPump, THREAD_PRIORITY_ABOVE_NORMAL);

	ResumeThread(this->p_hTcpPump);

	WaitForSingleObject(p_hTcpPumpEvent, INFINITE);
	CloseHandle(p_hTcpPumpEvent);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to stop the data pump thread. It should not be called directly.
*/
bool CSocketServer::p_StopTcpPump(void)
{
	DWORD dwExitCode = 0;

	if(this->p_bTcpPumpActive)
	{
		this->p_bShutdownTcpPump = true;

		while(GetExitCodeThread(this->p_hTcpPump, &dwExitCode))
		{
			if(dwExitCode != STILL_ACTIVE)
			{
				CloseHandle(this->p_hTcpPump);
				return true;
			}
			Sleep(1);
		}

		return this->p_ThrowError(24);
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This is the actual data pump procedure,
		it is called by the data pump thread procedure: p_TcpPumpThreadProc().
*/
unsigned int CSocketServer::p_TCPPump(void)
{
	struct timeval tTimeOut; // The TimeOut structure.
	tTimeOut.tv_sec = 1;
	tTimeOut.tv_usec = 0;

	fd_set fdExcept;
	fd_set fdRead;
	fd_set fdWrite;

	time_t ttNow;

	while(!this->p_bShutdownTcpPump)
	{
		int iClient = 0;

		// Zero the Read, Write and Except
		FD_ZERO(&fdRead);
		FD_ZERO(&fdWrite);
		FD_ZERO(&fdExcept);

		if(!this->p_bIsListeningServer)
		{
			while(this->p_iCurrentClients == 0)
			{
				if(this->p_bShutdownTcpPump)
				{
					return true;
				}
				Sleep(1);
			}
			if(this->p_bShutdownTcpPump)
			{
				return true;
			}
		}
		else {
			// Set LocalListenSocket socket to Read, Except
			FD_SET(this->p_ListenSocket, &fdRead);
			FD_SET(this->p_ListenSocket, &fdExcept);
		}

		// Set all connected clients to Read, Write and Except
		if(this->p_iCurrentClients > 0)
		{
			for(iClient = 0; iClient < this->p_iMaxClients; iClient++)
			{
				if( this->CCI[iClient].bConnected )
				{
					FD_SET(this->CCI[iClient].Socket, &fdRead);
					FD_SET(this->CCI[iClient].Socket, &fdWrite);
					FD_SET(this->CCI[iClient].Socket, &fdExcept);
				}
			}
		}

		// Socket Select...
		if(select(-1, &fdRead, &fdWrite, &fdExcept, &tTimeOut) == SOCKET_ERROR)
		{
			return this->p_ThrowError(25);
		}

		if(this->p_bIsListeningServer)
		{
			// Accept any incoming connections.
			if(FD_ISSET(this->p_ListenSocket, &fdRead))
			{
				//Try to lock because new connections are not as
				//	important as the performance of existing connections.
				if(this->TryLock())
				{
					this->p_Accept();
					this->Unlock();
				}
			}
		}

		// Send and receive any pending data.
		if(this->p_iCurrentClients > 0)
		{
			for(iClient = 0; iClient < this->p_iMaxClients; iClient++)
			{
				if(this->CCI[iClient].bConnected)
				{
					this->Lock();
					/*
						Send, receive then test for errors.

						Since we almost always send a response [in response] to received data,
							we want to perform the receive first, followed immediately by a send.
					*/

					if( FD_ISSET(this->CCI[iClient].Socket, &fdRead) )
					{
   						this->p_Receive(iClient);
					}
					if( FD_ISSET(this->CCI[iClient].Socket, &fdWrite) )
					{
   						this->p_Send(iClient);
					}
					if( FD_ISSET(this->CCI[iClient].Socket, &fdExcept) )
					{
						this->Disconnect(iClient);
					}

					if(this->p_dMaxInactivityTime > 0) //Is max inactivity time enabled?
					{
						if(!this->CCI[iClient].bReserved) //Don't disconnect clients that have been reserved.
						{
							//Has the client been inactive for too long (no send/receive for x seconds)?
							if(difftime(time(&ttNow), this->CCI[iClient].ttLastActivity) > this->p_dMaxInactivityTime)
							{
								this->Disconnect(iClient);
								this->p_ThrowError(2, iClient);
							}
						}
					}
					this->Unlock();

					// Disconnect any pending disconnects.
					if( this->CCI[iClient].bDisconnect && !this->CCI[iClient].bReserved  )
					{
						this->p_DisconnectClient(iClient);
					}
				}
			}
		}

		Sleep(1);
	}

	this->p_bTcpPumpActive = false;

	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This is the actual data pump thread.
*/
unsigned __stdcall p_TcpPumpThreadProc(void *pData)
{
	//---------------------(Initialize Thread [Begin])----------
	CSocketServer* pSock = (CSocketServer *) pData;

	pSock->p_bShutdownTcpPump = false;
	pSock->p_bTcpPumpActive = true;

	#ifdef _DEBUG
	#if defined(_MSC_VER) && (_MSC_VER >= 1400)
	pSock->p_SetThreadName("TCPPump");
	#endif
	#endif

	SetEvent(pSock->p_hTcpPumpEvent);
	//---------------------(Initialize Thread [END])----------

	_endthreadex(pSock->p_TCPPump());
	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
