///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _CSocketServer_StartStop_Cpp
#define _CSocketServer_StartStop_Cpp
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define _WIN32_WINNT 0x0500
#define WIN32_LEAN_AND_MEAN 

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <WinSock2.H>
#include <WinSock.H>
#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <Time.H>

#include "CSocketServer.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to Destroy all of the class variables.
		Call this function when you are done with the class.
*/
bool CSocketServer::Destroy(void)
{
	if(this->p_bActive)
	{
		if(this->p_iCurrentClients > 0) //Force disconnection.
		{
			for(int iClient = 0; iClient < this->p_iMaxClients; iClient++)
			{
				this->Lock();
				if(this->IsConnected(iClient))
				{
					this->p_DisconnectClient(iClient);
				}
				this->Unlock();
			}
		}

		this->Stop();
	}

	this->p_bActive = false;

	this->Free(&this->p_SendChunk);
	this->Free(&this->p_RecvChunk);

	this->Free(this->CCI);

	//Delete our Critical Section objects.
	DeleteCriticalSection(&this->p_TcpPumpLock);
	DeleteCriticalSection(&this->p_HostResolverLock);

	return this->p_StopWindowsSocket();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to Start the client or server.

	If the iListenPort is non-zero a listening port will be opened to
	accept incomming connections.
*/
bool CSocketServer::Start(void)
{
	if(this->pOnStart)
	{
		if(!this->pOnStart(this, this->p_iListenPort))
		{
			return false;
		}
	}
	
	//Are we going to be listening on any port?
	if(this->p_iListenPort > 0)
	{
		//Open our listening port.
		if(!this->p_OpenListenSocket(this->p_iListenPort))
		{
			return this->p_ThrowError(19);
		}
		this->p_bIsListeningServer = true;
	}

	if(!this->p_StartTcpPump())
	{
		return this->p_ThrowError(20);
	}

	this->p_bActive = true;
	this->AllowNewConnections(true);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to Stop the server or client. If there is a listening socket
	it will be shutdown and closed.

	bForce tells the stop() routine to stap the service even if the OnStop() routine returns false.
*/
bool CSocketServer::Stop(bool bForce)
{
	if(this->pOnStop)
	{
		if(!this->pOnStop(this) && !bForce)
		{
			return false;
		}
	}

	this->AllowNewConnections(false);

	if(this->p_iCurrentClients > 0) //Friendly disconnection.
	{
		for(int iClient = 0; iClient < this->p_iMaxClients; iClient++)
		{
			this->Lock();
			if(this->IsConnected(iClient))
			{
				this->Disconnect(iClient);
			}
			this->Unlock();
		}

		for(int iCount = SSSTOPFORCETIMEOUT; iCount > 0; iCount--)
		{
			this->Lock();
			if(this->p_iCurrentClients == 0)
			{
				this->Unlock();
				break;
			}
			this->Unlock();
			Sleep(1);
		}
	}
	
	if(!this->p_StopTcpPump())
	{
		return this->p_ThrowError(21);
	}

	if(this->p_bIsListeningServer)
	{
		if(!this->p_CloseListenSocket())
		{
			return this->p_ThrowError(22);
		}
		this->p_bIsListeningServer = false;
	}

	this->p_bActive = false;

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	This function is used to Stop the server or client. If there is a listening socket
	it will be shutdown and closed.
*/
bool CSocketServer::Stop(void)
{
	return this->Stop(false);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
